#include "../../include/freenet/periodicdbmaintenance.h"
#include "../../include/stringfunctions.h"

#ifdef XMEM
	#include <xmem.h>
#endif

PeriodicDBMaintenance::PeriodicDBMaintenance()
{
	m_check10mins.SetToGMTime();
	m_check30mins.SetToGMTime();
	m_check1hour.SetToGMTime();
	m_check6hours.SetToGMTime();
	m_check1day.SetToGMTime();

	// set back times so they will do their first maintenance soon ( within the next hour ) - stagger so not all run at once
	m_check30mins.Add(0,-5);
	m_check1hour.Add(0,-42);
	m_check6hours.Add(0,-1,-5);
	m_check1day.Add(0,0,-23);

}

void PeriodicDBMaintenance::Do10MinuteMaintenance()
{

	m_log->WriteLog(LogFile::LOGLEVEL_DEBUG,"PeriodicDBMaintenance::Do10MinuteMaintenance");
}

void PeriodicDBMaintenance::Do30MinuteMaintenance()
{

	m_log->WriteLog(LogFile::LOGLEVEL_DEBUG,"PeriodicDBMaintenance::Do30MinuteMaintenance");
}

void PeriodicDBMaintenance::Do1HourMaintenance()
{
	// recalculate all trust levels - this is CPU instensive
	// TODO - will probably have to change this to do 1 identity at a time as this locks that database for the duration
	m_db->Execute("UPDATE tblIdentity SET PeerMessageTrust=(SELECT PeerMessageTrust FROM vwCalculatedPeerTrust WHERE TargetIdentityID=IdentityID), PeerTrustListTrust=(SELECT PeerTrustListTrust FROM vwCalculatedPeerTrust WHERE TargetIdentityID=IdentityID);");

	m_log->WriteLog(LogFile::LOGLEVEL_DEBUG,"PeriodicDBMaintenance::Do1HourMaintenance");
}

void PeriodicDBMaintenance::Do6HourMaintenance()
{

	m_log->WriteLog(LogFile::LOGLEVEL_DEBUG,"PeriodicDBMaintenance::Do6HourMaintenance");
}

void PeriodicDBMaintenance::Do1DayMaintenance()
{
	DateTime date;

	// delete all puzzles 2 or more days old
	date.SetToGMTime();
	date.Add(0,0,0,-2);
	m_db->Execute("DELETE FROM tblIntroductionPuzzleInserts WHERE Day<='"+date.Format("%Y-%m-%d")+"';");
	m_db->Execute("DELETE FROM tblIntroductionPuzzleRequests WHERE Day<='"+date.Format("%Y-%m-%d")+"';");

	m_log->WriteLog(LogFile::LOGLEVEL_DEBUG,"PeriodicDBMaintenance::Do1DayMaintenance");

	// delete all identities we've never seen and were added more than 20 days ago
	date.SetToGMTime();
	date.Add(0,0,0,-20);
	m_db->Execute("DELETE FROM tblIdentity WHERE LastSeen IS NULL AND DateAdded<'"+date.Format("%Y-%m-%d")+"';");

	// delete old identity requests - we don't need them anymore
	date.SetToGMTime();
	date.Add(0,0,0,-2);
	m_db->Execute("DELETE FROM tblIdentityRequests WHERE Day<'"+date.Format("%Y-%m-%d")+"';");

	// delete old local identity inserts - we don't need them anymore
	date.SetToGMTime();
	date.Add(0,0,0,-2);
	m_db->Execute("DELETE FROM tblLocalIdentityInserts WHERE Day<'"+date.Format("%Y-%m-%d")+"';");

	// delete old message list inserts/requests - we don't need them anymore
	date.SetToGMTime();
	date.Add(0,0,0,-2);
	m_db->Execute("DELETE FROM tblMessageListInserts WHERE Day<'"+date.Format("%Y-%m-%d")+"';");
	m_db->Execute("DELETE FROM tblMessageListRequests WHERE Day<'"+date.Format("%Y-%m-%d")+"';");

	// delete old trust list inserts/requests - we don't need them anymore
	date.SetToGMTime();
	date.Add(0,0,0,-2);
	m_db->Execute("DELETE FROM tblTrustListInserts WHERE Day<'"+date.Format("%Y-%m-%d")+"';");
	m_db->Execute("DELETE FROM tblTrustListRequests WHERE Day<'"+date.Format("%Y-%m-%d")+"';");

	// delete trust lists from identities we aren't trusting anymore
	m_db->Execute("DELETE FROM tblPeerTrust WHERE IdentityID NOT IN (SELECT IdentityID FROM tblIdentity WHERE (LocalTrustListTrust>=(SELECT OptionValue FROM tblOption WHERE Option='MinLocalTrustListTrust')) AND (PeerTrustListTrust IS NULL OR PeerTrustListTrust>=(SELECT OptionValue FROM tblOption WHERE Option='MinPeerTrustListTrust')));");

	// remove identityid from messages where the identity has been deleted
	m_db->Execute("UPDATE tblMessage SET IdentityID=NULL WHERE IdentityID NOT IN (SELECT IdentityID FROM tblIdentity);");

	// try to re-attach messages from identities that were previously deleted, but have been since re-added
	// first get the names from messages that have a NULL IdentityID
	SQLite3DB::Statement st=m_db->Prepare("SELECT FromName FROM tblMessage WHERE IdentityID IS NULL GROUP BY FromName;");
	st.Step();
	while(st.RowReturned())
	{
		std::string name="";
		std::string publickey="";
		int identityid=0;
		st.ResultText(0,name);

		std::vector<std::string> parts;
		StringFunctions::Split(name,"@",parts);

		// find identities with this name
		SQLite3DB::Statement st2=m_db->Prepare("SELECT IdentityID,PublicKey FROM tblIdentity WHERE Name=?;");
		st2.Bind(0,name);
		st2.Step();
		while(st2.RowReturned())
		{
			publickey="";
			identityid=0;
			st2.ResultText(1,publickey);
			// check if public key matches 2nd part
			if(parts.size()>1 && publickey.find(parts[1])==4)
			{
				// we have the identity - so update the messages table with the identityid
				st2.ResultInt(0,identityid);

				SQLite3DB::Statement st3=m_db->Prepare("UPDATE tblMessage SET IdentityID=? WHERE Name=? AND IdentityID IS NULL;");
				st3.Bind(0,identityid);
				st3.Bind(1,name);
				st3.Step();
			}
			st2.Step();
		}

		st.Step();
	}

}

void PeriodicDBMaintenance::Process()
{
	DateTime now;
	now.SetToGMTime();
	
	if(m_check10mins<=(now-((1.0/1440.0)*10.0)))
	{
		Do10MinuteMaintenance();
		m_check10mins=now;
	}
	if(m_check30mins<=(now-((1.0/1440.0)*30.0)))
	{
		Do30MinuteMaintenance();
		m_check30mins=now;
	}
	if(m_check1hour<=(now-(1.0/24.0)))
	{
		Do1HourMaintenance();
		m_check1hour=now;
	}
	if(m_check6hours<=(now-(1.0/4.0)))
	{
		Do6HourMaintenance();
		m_check6hours=now;
	}
	if(m_check1day<=(now-(1.0)))
	{
		Do1DayMaintenance();
		m_check1day=now;
	}

}

void PeriodicDBMaintenance::RegisterWithThread(FreenetMasterThread *thread)
{
	thread->RegisterPeriodicProcessor(this);
}
